@@ -1,4 +1,4 @@
-/* $Header: /src/pub/tcsh/config_f.h,v 3.30 2004/08/08 06:42:28 christos Exp $ */
+/* $Header: /p/tcsh/cvsroot/tcsh/config_f.h,v 3.43 2010/02/09 20:28:17 christos Exp $ */
 /*
  * config_f.h -- configure various defines for tcsh
  *
@@ -50,18 +50,13 @@
  * WIDE_STRINGS	Represent strings using wide characters
  *		Allows proper function in multibyte encodings like UTF-8
  */
-#if defined (SHORT_STRINGS) && SIZEOF_WCHAR_T >= 4 && !defined (WINNT_NATIVE)
+#if defined (SHORT_STRINGS) && defined (NLS) && defined (HAVE_MBRTOWC) && !defined (WINNT_NATIVE) && !defined(_OSD_POSIX)
 # define WIDE_STRINGS
+# if SIZEOF_WCHAR_T < 4
+#  define UTF16_STRINGS
+# endif
 #endif
 
-/*
- * NLS:		Use Native Language System
- *		Routines like setlocale() are needed
- *		if you don't have <locale.h>, you don't want
- *		to define this.
- */
-#define NLS
-
 /*
  * NLS_CATALOGS:Use Native Language System catalogs for
  *		international messages.
@@ -69,7 +64,9 @@
  *		if you don't have <nl_types.h>, you don't want
  *		to define this.
  */
-#undef NLS_CATALOGS
+#if defined (NLS) && defined (HAVE_CATGETS)
+# define NLS_CATALOGS
+#endif
 
 /*
  * LOGINFIRST   Source ~/.login before ~/.cshrc
@@ -195,7 +192,7 @@
 # ifndef __GNUC__
 #  define RCSID(id) static char *rcsid = (id);
 # else
-#  define RCSID(id) static char *rcsid(const char *a) { return rcsid(a = id); }
+#  define RCSID(id) static const char rcsid[] __attribute__((__used__)) = (id);
 # endif /* !__GNUC__ */
 #else
 # define RCSID(id)	/* Nothing */
@@ -203,29 +200,25 @@
 
 /* Consistency checks */
 #ifdef WIDE_STRINGS
-# if SIZEOF_WCHAR_T < 4
-#  error "wchar_t must be at least 4 bytes for WIDE_STRINGS"
-# endif
-
 # ifdef WINNT_NATIVE
-#  error "WIDE_STRINGS cannot be used together with WINNT_NATIVE"
+    #error "WIDE_STRINGS cannot be used together with WINNT_NATIVE"
 # endif
 
 # ifndef SHORT_STRINGS
-#  error "SHORT_STRINGS must be defined if WIDE_STRINGS is defined"
+    #error "SHORT_STRINGS must be defined if WIDE_STRINGS is defined"
 # endif
 
 # ifndef NLS
-#  error "NLS must be defined if WIDE_STRINGS is defined"
+    #error "NLS must be defined if WIDE_STRINGS is defined"
 # endif
 
 # ifdef DSPMBYTE
-#  error "DSPMBYTE must not be defined if WIDE_STRINGS is defined"
+    #error "DSPMBYTE must not be defined if WIDE_STRINGS is defined"
 # endif
 #endif
 
 #if !defined (SHORT_STRINGS) && defined (DSPMBYTE)
-# error "SHORT_STRINGS must be defined if DSPMBYTE is defined"
+    #error "SHORT_STRINGS must be defined if DSPMBYTE is defined"
 #endif
 
 #endif /* _h_config_f */
