@@ -1,4 +1,4 @@
-/* $Header: /p/tcsh/cvsroot/tcsh/sh.c,v 3.140 2008/09/25 14:41:48 christos Exp $ */
+/* $Header: /p/tcsh/cvsroot/tcsh/sh.c,v 3.158 2010/05/12 15:35:37 christos Exp $ */
 /*
  * sh.c: Main shell routines
  */
@@ -39,7 +39,7 @@ char    copyright[] =
  All rights reserved.\n";
 #endif /* not lint */
 
-RCSID("$tcsh: sh.c,v 3.140 2008/09/25 14:41:48 christos Exp $")
+RCSID("$tcsh: sh.c,v 3.158 2010/05/12 15:35:37 christos Exp $")
 
 #include "tc.h"
 #include "ed.h"
@@ -139,6 +139,7 @@ struct saved_state {
     Char	  HIST;
     int	  cantell;
     struct Bin	  B;
+    int		  justpr;
 };
 
 static	int		  srccat	(Char *, Char *);
@@ -159,6 +160,42 @@ static	void		  st_restore	(void *);
 
 	int		  main		(int, char **);
 
+#ifndef LOCALEDIR
+#define LOCALEDIR "/usr/share/locale"
+#endif
+
+#ifdef NLS_CATALOGS
+static void
+add_localedir_to_nlspath(const char *path)
+{
+    static const char msgs[] = "/%L/LC_MESSAGES/%N.cat";
+    char *old = getenv("NLSPATH");
+    char *new;
+    size_t len = 0;
+
+    if (path == NULL)
+        return;
+
+    if (old != NULL)
+        len += strlen(old);
+
+    len += strlen(path) + sizeof(msgs);
+
+    new = xcalloc(len, 1);
+
+    if (old != NULL) {
+        (void)strncat(new, old, len);
+        (void)strncat(new, ":", len);
+    }
+
+    (void)strncat(new, path, len);
+    (void)strncat(new, msgs, len);
+
+    tsetenv(STRNLSPATH, str2short(new));
+    free(new);
+}
+#endif
+
 int
 main(int argc, char **argv)
 {
@@ -191,6 +228,13 @@ main(int argc, char **argv)
 # endif /* LC_CTYPE */
 #endif /* NLS */
 
+    STR_environ = blk2short(environ);
+    environ = short2blk(STR_environ);	/* So that we can free it */
+
+#ifdef NLS_CATALOGS
+    add_localedir_to_nlspath(LOCALEDIR);
+#endif
+
     nlsinit();
 
 #ifdef MALLOC_TRACE
@@ -246,7 +290,7 @@ main(int argc, char **argv)
 	t = t ? t + 1 : argv[0];
 	if (*t == '-') t++;
 	progname = strsave((t && *t) ? t : tcshstr);    /* never want a null */
-	tcsh = strcmp(progname, tcshstr) == 0;
+	tcsh = strncmp(progname, tcshstr, sizeof(tcshstr) - 1) == 0;
     }
 
     /*
@@ -262,14 +306,12 @@ main(int argc, char **argv)
     STR_SHELLPATH = SAVE(_PATH_CSHELL);
 # endif
 #endif
-    STR_environ = blk2short(environ);
-    environ = short2blk(STR_environ);	/* So that we can free it */
     STR_WORD_CHARS = SAVE(WORD_CHARS);
 
     HIST = '!';
     HISTSUB = '^';
-    PRCH = '>';
-    PRCHROOT = '#';
+    PRCH = tcsh ? '>' : '%';	/* to replace %# in $prompt for normal users */
+    PRCHROOT = '#';		/* likewise for root */
     word_chars = STR_WORD_CHARS;
     bslash_quote = 0;		/* PWP: do tcsh-style backslash quoting? */
 
@@ -318,7 +360,6 @@ main(int argc, char **argv)
 	setNS(STRloginsh);
     }
 
-    AsciiOnly = 1;
     NoNLSRebind = getenv("NOREBIND") != NULL;
 #ifdef NLS
 # ifdef SETLOCALEBUG
@@ -335,13 +376,19 @@ main(int argc, char **argv)
     fix_strcoll_bug();
 # endif /* STRCOLLBUG */
 
-    {
+    /*
+     * On solaris ISO8859-1 contains no printable characters in the upper half
+     * so we need to test only for MB_CUR_MAX == 1, otherwise for multi-byte
+     * locales we are always AsciiOnly == 0.
+     */
+    if (MB_CUR_MAX == 1) {
 	int     k;
 
-	for (k = 0200; k <= 0377 && !Isprint(CTL_ESC(k)); k++)
+	for (k = 0200; k <= 0377 && !isprint(CTL_ESC(k)); k++)
 	    continue;
-	AsciiOnly = MB_CUR_MAX == 1 && k > 0377;
-    }
+	AsciiOnly = k > 0377;
+    } else
+	AsciiOnly = 0;
 #else
     AsciiOnly = getenv("LANG") == NULL && getenv("LC_CTYPE") == NULL;
 #endif				/* NLS */
@@ -458,7 +505,7 @@ main(int argc, char **argv)
 	    else
 		cp2 = cp;
 	    if (!(((Strncmp(cp2, STRtty, 3) == 0) && Isalpha(cp2[3])) ||
-	          Strstr(cp, STRslptssl) != NULL)) {
+	          Strstr(cp, STRptssl) != NULL)) {
 		if (getenv("DISPLAY") == NULL) {
 		    /* NOT on X window shells */
 		    setcopy(STRautologout, STRdefautologout, VAR_READWRITE);
@@ -530,6 +577,12 @@ main(int argc, char **argv)
 	setv(STRoid, Itoa(oid, 0, 0), VAR_READWRITE);
 #endif /* apollo */
 
+	setv(STReuid, Itoa(euid, 0, 0), VAR_READWRITE);
+	if ((pw = xgetpwuid(euid)) == NULL)
+	    setcopy(STReuser, STRunknown, VAR_READWRITE);
+	else
+	    setcopy(STReuser, str2short(pw->pw_name), VAR_READWRITE);
+
 	setv(STRuid, Itoa(uid, 0, 0), VAR_READWRITE);
 
 	setv(STRgid, Itoa(gid, 0, 0), VAR_READWRITE);
@@ -541,7 +594,7 @@ main(int argc, char **argv)
 	else if (cln != NULL)
 	    setv(STRuser, quote(SAVE(cln)), VAR_READWRITE);
 	else if ((pw = xgetpwuid(uid)) == NULL)
-	    setcopy(STRuser, str2short("unknown"), VAR_READWRITE);
+	    setcopy(STRuser, STRunknown, VAR_READWRITE);
 	else
 	    setcopy(STRuser, str2short(pw->pw_name), VAR_READWRITE);
 	if (cln == NULL)
@@ -553,7 +606,7 @@ main(int argc, char **argv)
 	if (cgr != NULL)
 	    setv(STRgroup, quote(SAVE(cgr)), VAR_READWRITE);
 	else if ((gr = xgetgrgid(gid)) == NULL)
-	    setcopy(STRgroup, str2short("unknown"), VAR_READWRITE);
+	    setcopy(STRgroup, STRunknown, VAR_READWRITE);
 	else
 	    setcopy(STRgroup, str2short(gr->gr_name), VAR_READWRITE);
 	if (cgr == NULL)
@@ -572,7 +625,7 @@ main(int argc, char **argv)
 	    tsetenv(STRHOST, str2short(cbuff));
 	}
 	else
-	    tsetenv(STRHOST, str2short("unknown"));
+	    tsetenv(STRHOST, STRunknown);
     }
 
 
@@ -696,7 +749,7 @@ main(int argc, char **argv)
 	parseLS_COLORS(str2short(tcp));
 #endif /* COLOR_LS_F */
 
-    doldol = putn((int) getpid());	/* For $$ */
+    doldol = putn((tcsh_number_t)getpid());	/* For $$ */
 #ifdef WINNT_NATIVE
     {
 	char *tmp;
@@ -704,8 +757,8 @@ main(int argc, char **argv)
 	if ((tmp = getenv("TMP")) != NULL) {
 	    tmp = xasprintf("%s/%s", tmp, "sh");
 	    tmp2 = SAVE(tmp);
-	xfree(tmp);
-    }
+	    xfree(tmp);
+	}
 	else {
 	    tmp2 = SAVE(""); 
 	}
@@ -737,10 +790,11 @@ main(int argc, char **argv)
     /* PATCH IDEA FROM Issei.Suzuki VERY THANKS */
 #if defined(DSPMBYTE)
 #if defined(NLS) && defined(LC_CTYPE)
-    if (((tcp = setlocale(LC_CTYPE, NULL)) != NULL || (tcp = getenv("LANG")) != NULL) && !adrof(CHECK_MBYTEVAR)) {
+    if (((tcp = setlocale(LC_CTYPE, NULL)) != NULL || (tcp = getenv("LANG")) != NULL) && !adrof(CHECK_MBYTEVAR))
 #else
-    if ((tcp = getenv("LANG")) != NULL && !adrof(CHECK_MBYTEVAR)) {
+    if ((tcp = getenv("LANG")) != NULL && !adrof(CHECK_MBYTEVAR))
 #endif
+    {
 	autoset_dspmbyte(str2short(tcp));
     }
 #if defined(WINNT_NATIVE)
@@ -757,7 +811,7 @@ main(int argc, char **argv)
     }
     if (argc > 1 && strcmp(argv[1], "--help") == 0) {
 	xprintf("%S\n\n", varval(STRversion));
-	xprintf(CGETS(11, 8, HELP_STRING));
+	xprintf("%s", CGETS(11, 8, HELP_STRING));
 	xexit(0);
     }
     /*
@@ -911,6 +965,8 @@ main(int argc, char **argv)
 
 	    case ' ':
 	    case '\t':
+	    case '\r':
+	    case '\n':
 		/* 
 		 * for O/S's that don't do the argument parsing right in 
 		 * "#!/foo -f " scripts
@@ -1018,10 +1074,7 @@ main(int argc, char **argv)
      * Set up the prompt.
      */
     if (prompt) {
-	if (tcsh)
-	    setcopy(STRprompt, STRdeftcshprompt, VAR_READWRITE);
-	else
-	    setcopy(STRprompt, STRdefcshprompt, VAR_READWRITE);
+	setcopy(STRprompt, STRdefprompt, VAR_READWRITE);
 	/* that's a meta-questionmark */
 	setcopy(STRprompt2, STRmquestion, VAR_READWRITE);
 	setcopy(STRprompt3, STRKCORRECT, VAR_READWRITE);
@@ -1103,17 +1156,7 @@ main(int argc, char **argv)
 	    }
 #endif /* NeXT */
 #ifdef BSDJOBS			/* if we have tty job control */
-    retry:
-	    if ((tpgrp = tcgetpgrp(f)) != -1) {
-		if (tpgrp != shpgrp) {
-		    struct sigaction old;
-
-		    sigaction(SIGTTIN, NULL, &old);
-		    signal(SIGTTIN, SIG_DFL);
-		    (void) kill(0, SIGTTIN);
-		    sigaction(SIGTTIN, &old, NULL);
-		    goto retry;
-		}
+	    if (grabpgrp(f, shpgrp) != -1) {
 		/*
 		 * Thanks to Matt Day for the POSIX references, and to
 		 * Paul Close for the SGI clarification.
@@ -1171,8 +1214,9 @@ main(int argc, char **argv)
 	    if (tpgrp == -1) {
 	notty:
 	        xprintf(CGETS(11, 1, "Warning: no access to tty (%s).\n"),
-			strerror(errno));
-		xprintf(CGETS(11, 2, "Thus no job control in this shell.\n"));
+		    strerror(errno));
+		xprintf("%s",
+		    CGETS(11, 2, "Thus no job control in this shell.\n"));
 		/*
 		 * Fix from:Sakari Jalovaara <sja@sirius.hut.fi> if we don't
 		 * have access to tty, disable editing too
@@ -1299,6 +1343,8 @@ main(int argc, char **argv)
     /*
      * Mop-up.
      */
+    /* Take care of these (especially HUP) here instead of inside flush. */
+    handle_pending_signals();
     if (intty) {
 	if (loginsh) {
 	    xprintf("logout\n");
@@ -1480,6 +1526,7 @@ st_save(struct saved_state *st, int unit, int hflg, Char **al, Char **av)
     st->alvec		= alvec;
     st->onelflg		= onelflg;
     st->enterhist	= enterhist;
+    st->justpr		= justpr;
     if (hflg)
 	st->HIST	= HIST;
     else
@@ -1578,6 +1625,7 @@ st_restore(void *xst)
 	HIST	= st->HIST;
     enterhist	= st->enterhist;
     cantell	= st->cantell;
+    justpr	= st->justpr;
 
     if (st->argv != NULL)
 	setq(STRargv, st->argv, &shvhed, VAR_READWRITE);
@@ -1647,6 +1695,7 @@ goodbye(Char **v, struct command *c)
 	size_t omark;
 	sigset_t set;
 
+	sigemptyset(&set);
 	signal(SIGQUIT, SIG_IGN);
 	sigaddset(&set, SIGQUIT);
 	sigprocmask(SIG_UNBLOCK, &set, NULL);
@@ -1682,7 +1731,7 @@ void
 exitstat(void)
 {
 #ifdef PROF
-    monitor(0);
+    _mcleanup();
 #endif
     /*
      * Note that if STATUS is corrupted (i.e. getn bombs) then error will exit
@@ -1842,7 +1891,7 @@ void
 process(int catch)
 {
     jmp_buf_t osetexit;
-    /* PWP: This might get nuked my longjmp so don't make it a register var */
+    /* PWP: This might get nuked by longjmp so don't make it a register var */
     size_t omark;
     volatile int didexitset = 0;
 
@@ -2023,6 +2072,7 @@ process(int catch)
     cleanup_pop_mark(omark);
     resexit(osetexit);
     exitset--;
+    handle_pending_signals();
 }
 
 /*ARGSUSED*/
@@ -2148,6 +2198,7 @@ mailchk(void)
 
 	    if (mailcount == 0)
 		continue;
+	    (void)closedir(mailbox);
 
 	    if (cnt == 1)
 		xprintf(CGETS(11, 3, "You have %d mail messages.\n"),
@@ -2263,12 +2314,13 @@ xexit(int i)
 
     {
 	struct process *pp, *np;
-
+	pid_t mypid = getpid();
 	/* Kill all processes marked for hup'ing */
 	for (pp = proclist.p_next; pp; pp = pp->p_next) {
 	    np = pp;
-	    do 
-		if ((np->p_flags & PHUP) && np->p_jobid != shpgrp) {
+	    do
+		if ((np->p_flags & PHUP) && np->p_jobid != shpgrp &&
+		    np->p_parentid == mypid) {
 		    if (killpg(np->p_jobid, SIGHUP) != -1) {
 			/* In case the job was suspended... */
 #ifdef SIGCONT
@@ -2355,4 +2407,30 @@ record(void)
 	recdirs(NULL, adrof(STRsavedirs) != NULL);
 	rechist(NULL, adrof(STRsavehist) != NULL);
     }
+    displayHistStats("Exiting");	/* no-op unless DEBUG_HIST */
+}
+
+/*
+ * Grab the tty repeatedly, and give up if we are not in the correct
+ * tty process group.
+ */
+int
+grabpgrp(int fd, pid_t desired)
+{
+    struct sigaction old;
+    pid_t pgrp;
+    size_t i;
+
+    for (i = 0; i < 100; i++) {
+	if ((pgrp = tcgetpgrp(fd)) == -1)
+	    return -1;
+	if (pgrp == desired)
+	    return 0;
+	(void)sigaction(SIGTTIN, NULL, &old);
+	(void)signal(SIGTTIN, SIG_DFL);
+	(void)kill(0, SIGTTIN);
+	(void)sigaction(SIGTTIN, &old, NULL);
+    }
+    errno = EPERM;
+    return -1;
 }
