@@ -1,4 +1,4 @@
-/* $Header: /p/tcsh/cvsroot/tcsh/sh.exec.c,v 3.74 2007/05/31 08:26:12 corinna Exp $ */
+/* $Header: /p/tcsh/cvsroot/tcsh/sh.exec.c,v 3.77 2010/05/12 15:04:11 christos Exp $ */
 /*
  * sh.exec.c: Search, find, and execute a command!
  */
@@ -32,7 +32,7 @@
  */
 #include "sh.h"
 
-RCSID("$tcsh: sh.exec.c,v 3.74 2007/05/31 08:26:12 corinna Exp $")
+RCSID("$tcsh: sh.exec.c,v 3.77 2010/05/12 15:04:11 christos Exp $")
 
 #include "tc.h"
 #include "tw.h"
@@ -77,7 +77,7 @@ static Char *expath;		/* Path for exerr */
 /*
  * xhash is an array of hash buckets which are used to hash execs.  If
  * it is allocated (havhash true), then to tell if ``name'' is
- * (possibly) presend in the i'th component of the variable path, look
+ * (possibly) present in the i'th component of the variable path, look
  * at the [hashname(name)] bucket of size [hashwidth] bytes, in the [i
  * mod size*8]'th bit.  The cache size is defaults to a length of 1024
  * buckets, each 1 byte wide.  This implementation guarantees that
@@ -149,9 +149,9 @@ static	int 	iscommand	(Char *);
 void
 doexec(struct command *t, int do_glob)
 {
-    Char *dp, **pv, **av, *sav;
+    Char *dp, **pv, **opv, **av, *sav;
     struct varent *v;
-    int slash, gflag;
+    int slash, gflag, rehashed;
     int hashval, i;
     Char   *blk[2];
 
@@ -253,9 +253,9 @@ doexec(struct command *t, int do_glob)
      * command search.
      */
     if (v == NULL || v->vec == NULL || v->vec[0] == NULL || slash)
-	pv = justabs;
+	opv = justabs;
     else
-	pv = v->vec;
+	opv = v->vec;
     sav = Strspl(STRslash, *av);/* / command name for postpending */
 #ifndef VFORK
     cleanup_push(sav, xfree);
@@ -264,6 +264,9 @@ doexec(struct command *t, int do_glob)
 #endif /* VFORK */
     hashval = havhash ? hashname(*av) : 0;
 
+    rehashed = 0;
+retry:
+    pv = opv;
     i = 0;
 #ifdef VFORK
     hits++;
@@ -313,6 +316,11 @@ cont:
 #ifdef VFORK
     hits--;
 #endif /* VFORK */
+    if (adrof(STRautorehash) && !rehashed && havhash && opv != justabs) {
+	dohash(NULL, NULL);
+	rehashed = 1;
+	goto retry;
+    }
 #ifndef VFORK
     cleanup_until(sav);
 #else /* VFORK */
@@ -801,19 +809,23 @@ hashname(Char *cp)
 static int
 iscommand(Char *name)
 {
-    Char **pv;
+    Char **opv, **pv;
     Char *sav;
     struct varent *v;
     int slash = any(short2str(name), '/');
-    int hashval, i;
+    int hashval, rehashed, i;
 
     v = adrof(STRpath);
     if (v == NULL || v->vec == NULL || v->vec[0] == NULL || slash)
-	pv = justabs;
+	opv = justabs;
     else
-	pv = v->vec;
+	opv = v->vec;
     sav = Strspl(STRslash, name);	/* / command name for postpending */
     hashval = havhash ? hashname(name) : 0;
+
+    rehashed = 0;
+retry:
+    pv = opv;
     i = 0;
     do {
 	if (!slash && ABSOLUTEP(pv[0]) && havhash) {
@@ -842,6 +854,11 @@ cont:
 	pv++;
 	i++;
     } while (*pv);
+    if (adrof(STRautorehash) && !rehashed && havhash && opv != justabs) {
+	dohash(NULL, NULL);
+	rehashed = 1;
+	goto retry;
+    }
     xfree(sav);
     return 0;
 }
@@ -1063,10 +1080,10 @@ find_cmd(Char *cmd, int prt)
     const struct biltins *bptr;
     Char **pv;
     Char *sv;
-    int hashval, i, ex, rval = 0;
+    int hashval, rehashed, i, ex, rval = 0;
 
     if (prt && any(short2str(cmd), '/')) {
-	xprintf(CGETS(13, 7, "where: / in command makes no sense\n"));
+	xprintf("%s", CGETS(13, 7, "where: / in command makes no sense\n"));
 	return rval;
     }
 
@@ -1115,6 +1132,8 @@ find_cmd(Char *cmd, int prt)
     sv = Strspl(STRslash, cmd);
     cleanup_push(sv, xfree);
 
+    rehashed = 0;
+retry:
     for (pv = var->vec, i = 0; pv && *pv; pv++, i++) {
 	if (havhash && !eq(*pv, STRdot)) {
 #ifdef FASTHASH
@@ -1129,7 +1148,7 @@ find_cmd(Char *cmd, int prt)
 	ex = executable(*pv, sv, 0);
 #ifdef FASTHASH
 	if (!ex && (hashdebug & 2)) {
-	    xprintf(CGETS(13, 10, "hash miss: "));
+	    xprintf("%s", CGETS(13, 10, "hash miss: "));
 	    ex = 1;	/* Force printing */
 	}
 #endif /* FASTHASH */
@@ -1143,6 +1162,11 @@ find_cmd(Char *cmd, int prt)
 		return rval;
 	}
     }
+    if (adrof(STRautorehash) && !rehashed && havhash) {
+	dohash(NULL, NULL);
+	rehashed = 1;
+	goto retry;
+    }
     cleanup_until(sv);
     return rval;
 }
