@@ -1,4 +1,4 @@
-/* $Header: /src/pub/tcsh/sh.exp.c,v 3.42 2004/08/04 17:12:29 christos Exp $ */
+/* $Header: /p/tcsh/cvsroot/tcsh/sh.exp.c,v 3.55 2010/01/26 20:03:17 christos Exp $ */
 /*
  * sh.exp.c: Expression evaluations
  */
@@ -32,7 +32,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: sh.exp.c,v 3.42 2004/08/04 17:12:29 christos Exp $")
+RCSID("$tcsh: sh.exp.c,v 3.55 2010/01/26 20:03:17 christos Exp $")
 
 #include "tw.h"
 
@@ -57,36 +57,35 @@ RCSID("$Id: sh.exp.c,v 3.42 2004/08/04 17:12:29 christos Exp $")
 #define EQMATCH 7
 #define NOTEQMATCH 8
 
-static	int	 sh_access	__P((Char *, int));
-static	int	 exp1		__P((Char ***, bool));
-static	int	 exp2x		__P((Char ***, bool));
-static	int	 exp2a		__P((Char ***, bool));
-static	int	 exp2b		__P((Char ***, bool));
-static	int	 exp2c		__P((Char ***, bool));
-static	Char 	*exp3		__P((Char ***, bool));
-static	Char 	*exp3a		__P((Char ***, bool));
-static	Char 	*exp4		__P((Char ***, bool));
-static	Char 	*exp5		__P((Char ***, bool));
-static	Char 	*exp6		__P((Char ***, bool));
-static	void	 evalav		__P((Char **));
-static	int	 isa		__P((Char *, int));
-static	int	 egetn		__P((Char *));
-
+static	int	   sh_access	(const Char *, int);
+static	tcsh_number_t  exp1		(Char ***, int);
+static	tcsh_number_t  exp2x	(Char ***, int);
+static	tcsh_number_t  exp2a	(Char ***, int);
+static	tcsh_number_t  exp2b	(Char ***, int);
+static	tcsh_number_t  exp2c	(Char ***, int);
+static	Char 	  *exp3		(Char ***, int);
+static	Char 	  *exp3a	(Char ***, int);
+static	Char 	  *exp4		(Char ***, int);
+static	Char 	  *exp5		(Char ***, int);
+static	Char 	  *exp6		(Char ***, int);
+static	void	   evalav	(Char **);
+static	int	   isa		(Char *, int);
+static	tcsh_number_t  egetn	(const Char *);
 
 #ifdef EDEBUG
-static	void	 etracc		__P((char *, Char *, Char ***));
-static	void	 etraci		__P((char *, int, Char ***));
-#endif /* EDEBUG */
-
+static	void	   etracc	(const char *, const Char *, Char ***);
+static	void	   etraci	(const char *, tcsh_number_t, Char ***);
+#else /* !EDEBUG */
+#define etracc(A, B, C) ((void)0)
+#define etraci(A, B, C) ((void)0)
+#endif /* !EDEBUG */
 
 /*
  * shell access function according to POSIX and non POSIX
  * From Beto Appleton (beto@aixwiz.aix.ibm.com)
  */
 static int
-sh_access(fname, mode)
-    Char *fname;
-    int mode;
+sh_access(const Char *fname, int mode)
 {
 #if defined(POSIX) && !defined(USE_ACCESS)
     struct stat     statb;
@@ -147,23 +146,9 @@ sh_access(fname, mode)
 
 # ifdef NGROUPS_MAX
     else {
-#  if defined(__386BSD__) || defined(BSD4_4)
-    /*
-     * These two decided that setgroup() should take an array of int's
-     * and they define _SC_NGROUPS_MAX without having sysconf
-     */
-#   undef _SC_NGROUPS_MAX	
-#   if defined(__NetBSD__) || defined(__FreeBSD__) || defined(__bsdi__)
-#    define GID_T gid_t
-#   else
-#    define GID_T int
-#   endif
-#  else
-#   define GID_T gid_t
-#  endif /* __386BSD__ || BSD4_4 */
 	/* you can be in several groups */
 	long	n;
-	GID_T	*groups;
+	GETGROUPS_T *groups;
 
 	/*
 	 * Try these things to find a positive maximum groups value:
@@ -172,15 +157,15 @@ sh_access(fname, mode)
 	 *   3) getgroups(0, unused)
 	 * Then allocate and scan the groups array if one of these worked.
 	 */
-#  ifdef _SC_NGROUPS_MAX
+#  if defined (HAVE_SYSCONF) && defined (_SC_NGROUPS_MAX)
 	if ((n = sysconf(_SC_NGROUPS_MAX)) == -1)
 #  endif /* _SC_NGROUPS_MAX */
 	    n = NGROUPS_MAX;
 	if (n <= 0)
-	    n = getgroups(0, (GID_T *) NULL);
+	    n = getgroups(0, (GETGROUPS_T *) NULL);
 
 	if (n > 0) {
-	    groups = (GID_T *) xmalloc((size_t) (n * sizeof(GID_T)));
+	    groups = xmalloc(n * sizeof(*groups));
 	    n = getgroups((int) n, groups);
 	    while (--n >= 0)
 		if (groups[n] == statb.st_gid) {
@@ -198,148 +183,144 @@ sh_access(fname, mode)
 #endif /* !POSIX */
 }
 
-int
-expr(vp)
-    Char ***vp;
+tcsh_number_t
+expr(Char ***vp)
 {
     return (exp0(vp, 0));
 }
 
-int
-exp0(vp, ignore)
-    Char ***vp;
-    bool    ignore;
+tcsh_number_t
+exp0(Char ***vp, int ignore)
 {
-    int p1 = exp1(vp, ignore);
+    tcsh_number_t p1 = exp1(vp, ignore);
 
-#ifdef EDEBUG
     etraci("exp0 p1", p1, vp);
-#endif /* EDEBUG */
-    if (**vp && eq(**vp, STRor2)) {
+    while (**vp && eq(**vp, STRor2)) {
 	int p2;
 
 	(*vp)++;
-	p2 = exp0(vp, (ignore & TEXP_IGNORE) || p1);
-#ifdef EDEBUG
-	etraci("exp0 p2", p2, vp);
-#endif /* EDEBUG */
-	return (p1 || p2);
+
+	p2 = compat_expr ? 
+	    exp0(vp, (ignore & TEXP_IGNORE) || p1) :
+	    exp1(vp, (ignore & TEXP_IGNORE) || p1);
+	if (compat_expr || !(ignore & TEXP_IGNORE))
+	    p1 = (p1 || p2);
+	etraci("exp0 p1", p1, vp);
+	if (compat_expr)
+	    break;
     }
     return (p1);
 }
 
-static int
-exp1(vp, ignore)
-    Char ***vp;
-    bool    ignore;
+static tcsh_number_t
+exp1(Char ***vp, int ignore)
 {
-    int p1 = exp2x(vp, ignore);
+    tcsh_number_t p1 = exp2x(vp, ignore);
 
-#ifdef EDEBUG
     etraci("exp1 p1", p1, vp);
-#endif /* EDEBUG */
-    if (**vp && eq(**vp, STRand2)) {
-	int p2;
+    while (**vp && eq(**vp, STRand2)) {
+	tcsh_number_t p2;
 
 	(*vp)++;
-	p2 = exp1(vp, (ignore & TEXP_IGNORE) || !p1);
-#ifdef EDEBUG
+	p2 = compat_expr ?
+	    exp1(vp, (ignore & TEXP_IGNORE) || !p1) :
+	    exp2x(vp, (ignore & TEXP_IGNORE) || !p1);
+
 	etraci("exp1 p2", p2, vp);
-#endif /* EDEBUG */
-	return (p1 && p2);
+	if (compat_expr || !(ignore & TEXP_IGNORE))
+	    p1 = (p1 && p2);
+	etraci("exp1 p1", p1, vp);
+	if (compat_expr)
+	    break;
     }
     return (p1);
 }
 
-static int
-exp2x(vp, ignore)
-    Char ***vp;
-    bool    ignore;
+static tcsh_number_t
+exp2x(Char ***vp, int ignore)
 {
-    int p1 = exp2a(vp, ignore);
+    tcsh_number_t p1 = exp2a(vp, ignore);
 
-#ifdef EDEBUG
-    etraci("exp3 p1", p1, vp);
-#endif /* EDEBUG */
-    if (**vp && eq(**vp, STRor)) {
-	int p2;
+    etraci("exp2x p1", p1, vp);
+    while (**vp && eq(**vp, STRor)) {
+	tcsh_number_t p2;
 
 	(*vp)++;
-	p2 = exp2x(vp, ignore);
-#ifdef EDEBUG
-	etraci("exp3 p2", p2, vp);
-#endif /* EDEBUG */
-	return (p1 | p2);
+	p2 = compat_expr ?
+	    exp2x(vp, ignore) :
+	    exp2a(vp, ignore);
+	etraci("exp2x p2", p2, vp);
+	if (compat_expr || !(ignore & TEXP_IGNORE))
+		p1 = (p1 | p2);
+	etraci("exp2x p1", p1, vp);
+	if (compat_expr)
+	    break;
     }
     return (p1);
 }
 
-static int
-exp2a(vp, ignore)
-    Char ***vp;
-    bool    ignore;
+static tcsh_number_t
+exp2a(Char ***vp, int ignore)
 {
-    int p1 = exp2b(vp, ignore);
+    tcsh_number_t p1 = exp2b(vp, ignore);
 
-#ifdef EDEBUG
     etraci("exp2a p1", p1, vp);
-#endif /* EDEBUG */
-    if (**vp && eq(**vp, STRcaret)) {
-	int p2;
+    while (**vp && eq(**vp, STRcaret)) {
+	tcsh_number_t p2;
 
 	(*vp)++;
-	p2 = exp2a(vp, ignore);
-#ifdef EDEBUG
+	p2 = compat_expr ?
+	    exp2a(vp, ignore) :
+	    exp2b(vp, ignore);
 	etraci("exp2a p2", p2, vp);
-#endif /* EDEBUG */
-	return (p1 ^ p2);
+	if (compat_expr || !(ignore & TEXP_IGNORE))
+	    p1 = (p1 ^ p2);
+	etraci("exp2a p1", p1, vp);
+	if (compat_expr)
+	    break;
     }
     return (p1);
 }
 
-static int
-exp2b(vp, ignore)
-    Char ***vp;
-    bool    ignore;
+static tcsh_number_t
+exp2b(Char ***vp, int ignore)
 {
-    int p1 = exp2c(vp, ignore);
+    tcsh_number_t p1 = exp2c(vp, ignore);
 
-#ifdef EDEBUG
     etraci("exp2b p1", p1, vp);
-#endif /* EDEBUG */
-    if (**vp && eq(**vp, STRand)) {
-	int p2;
+    while (**vp && eq(**vp, STRand)) {
+	tcsh_number_t p2;
 
 	(*vp)++;
-	p2 = exp2b(vp, ignore);
-#ifdef EDEBUG
+	p2 = compat_expr ?
+	    exp2b(vp, ignore) :
+	    exp2c(vp, ignore);
 	etraci("exp2b p2", p2, vp);
-#endif /* EDEBUG */
-	return (p1 & p2);
+	if (compat_expr || !(ignore & TEXP_IGNORE))
+	    p1 = (p1 & p2);
+	etraci("exp2b p1", p1, vp);
+	if (compat_expr)
+	    break;
     }
     return (p1);
 }
 
-static int
-exp2c(vp, ignore)
-    Char ***vp;
-    bool    ignore;
+static tcsh_number_t
+exp2c(Char ***vp, int ignore)
 {
     Char *p1 = exp3(vp, ignore);
     Char *p2;
-    int i;
+    tcsh_number_t i;
 
-#ifdef EDEBUG
+    cleanup_push(p1, xfree);
     etracc("exp2c p1", p1, vp);
-#endif /* EDEBUG */
     if ((i = isa(**vp, EQOP)) != 0) {
 	(*vp)++;
 	if (i == EQMATCH || i == NOTEQMATCH)
 	    ignore |= TEXP_NOGLOB;
 	p2 = exp3(vp, ignore);
-#ifdef EDEBUG
+	cleanup_push(p2, xfree);
 	etracc("exp2c p2", p2, vp);
-#endif /* EDEBUG */
 	if (!(ignore & TEXP_IGNORE))
 	    switch (i) {
 
@@ -359,35 +340,32 @@ exp2c(vp, ignore)
 		i = !Gmatch(p1, p2);
 		break;
 	    }
-	xfree((ptr_t) p1);
-	xfree((ptr_t) p2);
+	cleanup_until(p1);
 	return (i);
     }
     i = egetn(p1);
-    xfree((ptr_t) p1);
+    cleanup_until(p1);
     return (i);
 }
 
 static Char *
-exp3(vp, ignore)
-    Char ***vp;
-    bool    ignore;
+exp3(Char ***vp, int ignore)
 {
     Char *p1, *p2;
-    int i;
+    tcsh_number_t i;
 
     p1 = exp3a(vp, ignore);
-#ifdef EDEBUG
     etracc("exp3 p1", p1, vp);
-#endif /* EDEBUG */
-    if ((i = isa(**vp, RELOP)) != 0) {
+    while ((i = isa(**vp, RELOP)) != 0) {
 	(*vp)++;
 	if (**vp && eq(**vp, STRequal))
 	    i |= 1, (*vp)++;
-	p2 = exp3(vp, ignore);
-#ifdef EDEBUG
+	cleanup_push(p1, xfree);
+	p2 = compat_expr ?
+	    exp3(vp, ignore) :
+	    exp3a(vp, ignore);
+	cleanup_push(p2, xfree);
 	etracc("exp3 p2", p2, vp);
-#endif /* EDEBUG */
 	if (!(ignore & TEXP_IGNORE))
 	    switch (i) {
 
@@ -407,62 +385,61 @@ exp3(vp, ignore)
 		i = egetn(p1) <= egetn(p2);
 		break;
 	    }
-	xfree((ptr_t) p1);
-	xfree((ptr_t) p2);
-	return (putn(i));
+	cleanup_until(p1);
+	p1 = putn(i);
+	etracc("exp3 p1", p1, vp);
+	if (compat_expr)
+	    break;
     }
     return (p1);
 }
 
 static Char *
-exp3a(vp, ignore)
-    Char ***vp;
-    bool    ignore;
+exp3a(Char ***vp, int ignore)
 {
-    Char *p1, *p2, *op;
-    int i;
+    Char *p1, *p2;
+    const Char *op;
+    tcsh_number_t i;
 
     p1 = exp4(vp, ignore);
-#ifdef EDEBUG
     etracc("exp3a p1", p1, vp);
-#endif /* EDEBUG */
     op = **vp;
     if (op && any("<>", op[0]) && op[0] == op[1]) {
 	(*vp)++;
-	p2 = exp3a(vp, ignore);
-#ifdef EDEBUG
+	cleanup_push(p1, xfree);
+	p2 = compat_expr ?
+	    exp3a(vp, ignore) :
+	    exp4(vp, ignore);
+	cleanup_push(p2, xfree);
 	etracc("exp3a p2", p2, vp);
-#endif /* EDEBUG */
 	if (op[0] == '<')
 	    i = egetn(p1) << egetn(p2);
 	else
 	    i = egetn(p1) >> egetn(p2);
-	xfree((ptr_t) p1);
-	xfree((ptr_t) p2);
-	return (putn(i));
+	cleanup_until(p1);
+	p1 = putn(i);
+	etracc("exp3a p1", p1, vp);
     }
     return (p1);
 }
 
 static Char *
-exp4(vp, ignore)
-    Char ***vp;
-    bool    ignore;
+exp4(Char ***vp, int ignore)
 {
     Char *p1, *p2;
-    int i = 0;
+    tcsh_number_t i = 0;
 
     p1 = exp5(vp, ignore);
-#ifdef EDEBUG
     etracc("exp4 p1", p1, vp);
-#endif /* EDEBUG */
-    if (isa(**vp, ADDOP)) {
-	Char *op = *(*vp)++;
+    while (isa(**vp, ADDOP)) {
+	const Char *op = *(*vp)++;
 
-	p2 = exp4(vp, ignore);
-#ifdef EDEBUG
+	cleanup_push(p1, xfree);
+	p2 = compat_expr ?
+	    exp4(vp, ignore) :
+	    exp5(vp, ignore);
+	cleanup_push(p2, xfree);
 	etracc("exp4 p2", p2, vp);
-#endif /* EDEBUG */
 	if (!(ignore & TEXP_IGNORE))
 	    switch (op[0]) {
 
@@ -474,39 +451,41 @@ exp4(vp, ignore)
 		i = egetn(p1) - egetn(p2);
 		break;
 	    }
-	xfree((ptr_t) p1);
-	xfree((ptr_t) p2);
-	return (putn(i));
+	cleanup_until(p1);
+	p1 = putn(i);
+	etracc("exp4 p1", p1, vp);
+	if (compat_expr)
+	    break;
     }
     return (p1);
 }
 
 static Char *
-exp5(vp, ignore)
-    Char ***vp;
-    bool    ignore;
+exp5(Char ***vp, int ignore)
 {
     Char *p1, *p2;
-    int i = 0;
+    tcsh_number_t i = 0;
 
     p1 = exp6(vp, ignore);
-#ifdef EDEBUG
     etracc("exp5 p1", p1, vp);
-#endif /* EDEBUG */
 
-    if (isa(**vp, MULOP)) {
-	Char *op = *(*vp)++;
-	if ((ignore & TEXP_NOGLOB) != 0) 
-	    /* 
+    while (isa(**vp, MULOP)) {
+	const Char *op = *(*vp)++;
+	if ((ignore & TEXP_NOGLOB) != 0) {
+	    /*
 	     * We are just trying to get the right side of
-	     * a =~ or !~ operator 
+	     * a =~ or !~ operator
 	     */
+	    xfree(p1);
 	    return Strsave(op);
+	}
 
-	p2 = exp5(vp, ignore);
-#ifdef EDEBUG
+	cleanup_push(p1, xfree);
+	p2 = compat_expr ? 
+	    exp5(vp, ignore) :
+	    exp6(vp, ignore);
+	cleanup_push(p2, xfree);
 	etracc("exp5 p2", p2, vp);
-#endif /* EDEBUG */
 	if (!(ignore & TEXP_IGNORE))
 	    switch (op[0]) {
 
@@ -528,19 +507,20 @@ exp5(vp, ignore)
 		i = egetn(p1) % i;
 		break;
 	    }
-	xfree((ptr_t) p1);
-	xfree((ptr_t) p2);
-	return (putn(i));
+	cleanup_until(p1);
+	p1 = putn(i);
+	etracc("exp5 p1", p1, vp);
+	if (compat_expr)
+	    break;
     }
     return (p1);
 }
 
 static Char *
-exp6(vp, ignore)
-    Char ***vp;
-    bool    ignore;
+exp6(Char ***vp, int ignore)
 {
-    int     ccode, i = 0;
+    tcsh_number_t ccode;
+    tcsh_number_t i = 0;
     Char *cp;
 
     if (**vp == 0)
@@ -548,30 +528,26 @@ exp6(vp, ignore)
     if (eq(**vp, STRbang)) {
 	(*vp)++;
 	cp = exp6(vp, ignore);
-#ifdef EDEBUG
+	cleanup_push(cp, xfree);
 	etracc("exp6 ! cp", cp, vp);
-#endif /* EDEBUG */
 	i = egetn(cp);
-	xfree((ptr_t) cp);
+	cleanup_until(cp);
 	return (putn(!i));
     }
     if (eq(**vp, STRtilde)) {
 	(*vp)++;
 	cp = exp6(vp, ignore);
-#ifdef EDEBUG
+	cleanup_push(cp, xfree);
 	etracc("exp6 ~ cp", cp, vp);
-#endif /* EDEBUG */
 	i = egetn(cp);
-	xfree((ptr_t) cp);
+	cleanup_until(cp);
 	return (putn(~i));
     }
     if (eq(**vp, STRLparen)) {
 	(*vp)++;
 	ccode = exp0(vp, ignore);
-#ifdef EDEBUG
 	etraci("exp6 () ccode", ccode, vp);
-#endif /* EDEBUG */
-	if (*vp == 0 || **vp == 0 || ***vp != ')')
+	if (**vp == 0 || ***vp != ')')
 	    stderror(ERR_NAME | ERR_EXPRESSION);
 	(*vp)++;
 	return (putn(ccode));
@@ -598,16 +574,15 @@ exp6(vp, ignore)
 	if (ignore & TEXP_IGNORE)
 	    return (Strsave(STRNULL));
 	psavejob();
+	cleanup_push(&faket, psavejob_cleanup); /* faket is only a marker */
 	if (pfork(&faket, -1) == 0) {
 	    *--(*vp) = 0;
 	    evalav(v);
 	    exitstat();
 	}
 	pwait();
-	prestjob();
-#ifdef EDEBUG
+	cleanup_until(&faket);
 	etraci("exp6 {} status", egetn(varval(STRstatus)), vp);
-#endif /* EDEBUG */
 	return (putn(egetn(varval(STRstatus)) == 0));
     }
     if (isa(**vp, ANYOP))
@@ -621,9 +596,7 @@ exp6(vp, ignore)
 #define FILEVALS  "ZAMCDIUGNFPL"
     if (*cp == '-' && (any(FILETESTS, cp[1]) || any(FILEVALS, cp[1])))
         return(filetest(cp, vp, ignore));
-#ifdef EDEBUG
     etracc("exp6 default", cp, vp);
-#endif /* EDEBUG */
     return (ignore & TEXP_NOGLOB ? Strsave(cp) : globone(cp, G_APPEND));
 }
 
@@ -633,9 +606,7 @@ exp6(vp, ignore)
  * From: John Rowe <rowe@excc.exeter.ac.uk>
  */
 Char *
-filetest(cp, vp, ignore)
-    Char *cp, ***vp;
-    bool ignore;
+filetest(Char *cp, Char ***vp, int ignore)
 {
 #ifdef convex
     struct cvxstat stb, *st = NULL;
@@ -656,9 +627,9 @@ filetest(cp, vp, ignore)
     char *filnam;
 #endif /* S_IFLNK */
 
-    int i = 0;
+    tcsh_number_t i = 0;
     unsigned pmask = 0xffff;
-    bool altout = 0;
+    int altout = 0;
     Char *ft = cp, *dp, *ep, *strdev, *strino, *strF, *str, valtest = '\0',
     *errval = STR0;
     char *string, string0[8];
@@ -710,6 +681,7 @@ filetest(cp, vp, ignore)
     if (ignore & TEXP_IGNORE)
 	return (Strsave(STRNULL));
     ep = globone(dp, G_APPEND);
+    cleanup_push(ep, xfree);
     ft = &cp[1];
     do 
 	switch (*ft) {
@@ -746,7 +718,7 @@ filetest(cp, vp, ignore)
 		if (!lst) {
 		    lst = &lstb;
 		    if (TCSH_LSTAT(short2str(ep), lst) == -1) {
-			xfree((ptr_t) ep);
+			cleanup_until(ep);
 			return (Strsave(errval));
 		    }
 		}
@@ -761,7 +733,7 @@ filetest(cp, vp, ignore)
 		if (!st) {
 		    st = &stb;
 		    if (TCSH_STAT(short2str(ep), st) == -1) {
-			xfree((ptr_t) ep);
+			cleanup_until(ep);
 			return (Strsave(errval));
 		    }
 		}
@@ -880,22 +852,22 @@ filetest(cp, vp, ignore)
 		 */
 
 	    case 'D':
-		i = (int) st->st_dev;
+		i = (tcsh_number_t) st->st_dev;
 		break;
 
 	    case 'I':
-		i = (int) st->st_ino;
+		i = (tcsh_number_t) st->st_ino;
 		break;
 		
 	    case 'F':
 		strdev = putn( (int) st->st_dev);
 		strino = putn( (int) st->st_ino);
-		strF = (Char *) xmalloc((size_t) (2 + Strlen(strdev) + 
-					 Strlen(strino)) * sizeof(Char));
+		strF = xmalloc((2 + Strlen(strdev) + Strlen(strino))
+			       * sizeof(Char));
 		(void) Strcat(Strcat(Strcpy(strF, strdev), STRcolon), strino);
-		xfree((ptr_t) strdev);
-		xfree((ptr_t) strino);
-		xfree((ptr_t) ep);
+		xfree(strdev);
+		xfree(strino);
+		cleanup_until(ep);
 		return(strF);
 		
 	    case 'L':
@@ -905,23 +877,10 @@ filetest(cp, vp, ignore)
 		}
 #ifdef S_ISLNK
 		filnam = short2str(ep);
-#ifdef PATH_MAX
-# define MY_PATH_MAX PATH_MAX
-#else /* !PATH_MAX */
-/* 
- * I can't think of any more sensible alterative; readlink doesn't give 
- * us an errno if the buffer isn't large enough :-(
- */
-# define MY_PATH_MAX  2048
-#endif /* PATH_MAX */
-		i = readlink(filnam, string = (char *) 
-		      xmalloc((size_t) (1 + MY_PATH_MAX) * sizeof(char)),
-			MY_PATH_MAX);
-		if (i >= 0 && i <= MY_PATH_MAX)
-		    string[i] = '\0'; /* readlink does not null terminate */
-		strF = (i < 0) ? errval : str2short(string);
-		xfree((ptr_t) string);
-		xfree((ptr_t) ep);
+		string = areadlink(filnam);
+		strF = string == NULL ? errval : str2short(string);
+		xfree(string);
+		cleanup_until(ep);
 		return(Strsave(strF));
 
 #else /* !S_ISLNK */
@@ -931,7 +890,7 @@ filetest(cp, vp, ignore)
 		
 
 	    case 'N':
-		i = (int) st->st_nlink;
+		i = (tcsh_number_t) st->st_nlink;
 		break;
 
 	    case 'P':
@@ -941,27 +900,27 @@ filetest(cp, vp, ignore)
 		    ((S_IRWXU|S_IRWXG|S_IRWXO|S_ISUID|S_ISGID) & st->st_mode));
 		if (altout && *string != '0')
 		    *--string = '0';
-		xfree((ptr_t) ep);
+		cleanup_until(ep);
 		return(Strsave(str2short(string)));
 
 	    case 'U':
-		if (altout && (pw = getpwuid(st->st_uid))) {
-		    xfree((ptr_t) ep);
+		if (altout && (pw = xgetpwuid(st->st_uid))) {
+		    cleanup_until(ep);
 		    return(Strsave(str2short(pw->pw_name)));
 		}
-		i = (int) st->st_uid;
+		i = (tcsh_number_t) st->st_uid;
 		break;
 
 	    case 'G':
-		if ( altout && (gr = getgrgid(st->st_gid))) {
-		    xfree((ptr_t) ep);
+		if (altout && (gr = xgetgrgid(st->st_gid))) {
+		    cleanup_until(ep);
 		    return(Strsave(str2short(gr->gr_name)));
 		}
-		i = (int) st->st_gid;
+		i = (tcsh_number_t) st->st_gid;
 		break;
 
 	    case 'Z':
-		i = (int) st->st_size;
+		i = (tcsh_number_t) st->st_size;
 		break;
 
 	    case 'A': case 'M': case 'C':
@@ -971,38 +930,34 @@ filetest(cp, vp, ignore)
 		    strF = str2short(ctime(&footime));
 		    if ((str = Strchr(strF, '\n')) != NULL)
 			*str = (Char) '\0';
-		    xfree((ptr_t) ep);
+		    cleanup_until(ep);
 		    return(Strsave(strF));
 		}
-		i = (int) footime;
+		i = (tcsh_number_t) footime;
 		break;
 
 	    }
 	}
     while (*++ft && i);
-#ifdef EDEBUG
     etraci("exp6 -? i", i, vp);
-#endif /* EDEBUG */
-    xfree((ptr_t) ep);
+    cleanup_until(ep);
     return (putn(i));
 }
 
 
 static void
-evalav(v)
-    Char **v;
+evalav(Char **v)
 {
     struct wordent paraml1;
     struct wordent *hp = &paraml1;
     struct command *t;
     struct wordent *wdp = hp;
 
-    set(STRstatus, Strsave(STR0), VAR_READWRITE);
+    setcopy(STRstatus, STR0, VAR_READWRITE);
     hp->prev = hp->next = hp;
     hp->word = STRNULL;
     while (*v) {
-	struct wordent *new =
-	(struct wordent *) xcalloc(1, sizeof *wdp);
+	struct wordent *new = xcalloc(1, sizeof *wdp);
 
 	new->prev = wdp;
 	new->next = hp;
@@ -1011,18 +966,18 @@ evalav(v)
 	wdp->word = Strsave(*v++);
     }
     hp->prev = wdp;
+    cleanup_push(&paraml1, lex_cleanup);
     alias(&paraml1);
     t = syntax(paraml1.next, &paraml1, 0);
+    cleanup_push(t, syntax_cleanup);
     if (seterr)
 	stderror(ERR_OLD);
     execute(t, -1, NULL, NULL, TRUE);
-    freelex(&paraml1), freesyn(t);
+    cleanup_until(&paraml1);
 }
 
 static int
-isa(cp, what)
-    Char *cp;
-    int what;
+isa(Char *cp, int what)
 {
     if (cp == 0)
 	return ((what & RESTOP) != 0);
@@ -1070,9 +1025,8 @@ isa(cp, what)
     return (0);
 }
 
-static int
-egetn(cp)
-    Char *cp;
+static tcsh_number_t
+egetn(const Char *cp)
 {
     if (*cp && *cp != '-' && !Isdigit(*cp))
 	stderror(ERR_NAME | ERR_EXPRESSION);
@@ -1083,22 +1037,20 @@ egetn(cp)
 
 #ifdef EDEBUG
 static void
-etraci(str, i, vp)
-    char   *str;
-    int     i;
-    Char ***vp;
+etraci(const char *str, tcsh_number_t i, Char ***vp)
 {
-    xprintf("%s=%d\t", str, i);
+#ifdef HAVE_LONG_LONG
+    xprintf("%s=%lld\t", str, i);
+#else
+    xprintf("%s=%ld\t", str, i);
+#endif
     blkpr(*vp);
     xputchar('\n');
 }
 static void
-etracc(str, cp, vp)
-    char   *str;
-    Char   *cp;
-    Char ***vp;
+etracc(const char *str, const Char *cp, Char ***vp)
 {
-    xprintf("%s=%s\t", str, cp);
+    xprintf("%s=%S\t", str, cp);
     blkpr(*vp);
     xputchar('\n');
 }
