@@ -1,4 +1,4 @@
-/* $Header: /p/tcsh/cvsroot/tcsh/sh.func.c,v 3.147 2008/09/25 14:41:48 christos Exp $ */
+/* $Header: /p/tcsh/cvsroot/tcsh/sh.func.c,v 3.156 2010/05/08 00:34:41 christos Exp $ */
 /*
  * sh.func.c: csh builtin functions
  */
@@ -32,7 +32,7 @@
  */
 #include "sh.h"
 
-RCSID("$tcsh: sh.func.c,v 3.147 2008/09/25 14:41:48 christos Exp $")
+RCSID("$tcsh: sh.func.c,v 3.156 2010/05/08 00:34:41 christos Exp $")
 
 #include "ed.h"
 #include "tw.h"
@@ -62,6 +62,7 @@ static	void	doagain		(void);
 static  const char *isrchx	(int);
 static	void	search		(int, int, Char *);
 static	int	getword		(struct Strbuf *);
+static	struct wordent	*histgetword	(struct wordent *);
 static	void	toend		(void);
 static	void	xecho		(int, Char **);
 static	int	islocale_var	(Char *);
@@ -754,6 +755,7 @@ search(int type, int level, Char *goal)
     Char *cp;
     struct whyle *wp;
     int wlevel = 0;
+    struct wordent *histent = NULL, *ohistent = NULL;
 
     Stype = type;
     Sgoal = goal;
@@ -765,11 +767,28 @@ search(int type, int level, Char *goal)
     }
     cleanup_push(&word, Strbuf_cleanup);
     do {
+	    
+	if (intty) {
+	    histent = xmalloc(sizeof(*histent));
+	    ohistent = xmalloc(sizeof(*histent));
+	    ohistent->word = STRNULL;
+	    ohistent->next = histent;
+	    histent->prev = ohistent;
+	}
+
 	if (intty && fseekp == feobp && aret == TCSH_F_SEEK)
 	    printprompt(1, isrchx(type == TC_BREAK ? zlast : type));
 	/* xprintf("? "), flush(); */
 	(void) getword(&word);
 	Strbuf_terminate(&word);
+
+	if (intty && Strlen(word.s) > 0) {
+	    histent->word = Strsave(word.s);
+	    histent->next = xmalloc(sizeof(*histent));
+	    histent->next->prev = histent;
+	    histent = histent->next;
+	}
+
 	switch (srchx(word.s)) {
 
 	case TC_ELSE:
@@ -856,12 +875,126 @@ search(int type, int level, Char *goal)
 		level = -1;
 	    break;
 	}
-	(void) getword(NULL);
+	if (intty) {
+	    ohistent->prev = histgetword(histent);
+	    ohistent->prev->next = ohistent;
+	    savehist(ohistent, 0);
+	    freelex(ohistent);
+	    xfree(ohistent);
+	} else 
+	    (void) getword(NULL);
     } while (level >= 0);
  end:
     cleanup_until(&word);
 }
 
+static struct wordent *
+histgetword(struct wordent *histent) 
+{
+    int found = 0, first;
+    eChar c, d;
+    int e;
+    struct Strbuf *tmp;
+    tmp = xmalloc(sizeof(*tmp));
+    tmp->size = 0;
+    tmp->s = NULL;
+    c = readc(1);
+    d = 0;
+    e = 0;
+    for (;;) {
+	tmp->len = 0;
+	Strbuf_terminate (tmp);
+	while (c == ' ' || c == '\t')
+	    c = readc(1);
+	if (c == '#')
+	    do
+		c = readc(1);
+	    while (c != CHAR_ERR && c != '\n');
+	if (c == CHAR_ERR)
+	    goto past;
+	if (c == '\n') 
+	    goto nl;
+	unreadc(c);
+	found = 1;
+	first = 1;
+	do {
+	    e = (c == '\\');
+	    c = readc(1);
+	    if (c == '\\' && !e) {
+		if ((c = readc(1)) == '\n') {
+		    e = 1;
+		    c = ' ';
+		} else {
+		    unreadc(c);
+		    c = '\\';
+		}
+	    }
+	    if ((c == '\'' || c == '"') && !e) {
+		if (d == 0)
+		    d = c;
+		else if (d == c)
+		    d = 0;
+	    }
+	    if (c == CHAR_ERR)
+		goto past;
+	    
+	    Strbuf_append1(tmp, (Char) c);
+	    
+	    if (!first && !d && c == '(' && !e) {
+		break;
+	    }
+	    first = 0;
+	} while (d || e || (c != ' ' && c != '\t' && c != '\n'));
+	tmp->len--;
+	if (tmp->len) {
+	    Strbuf_terminate(tmp);
+	    histent->word = Strsave(tmp->s);
+	    histent->next = xmalloc(sizeof (*histent));
+	    histent->next->prev = histent;
+	    histent = histent->next;
+	}
+	if (c == '\n') {
+	nl:
+	    tmp->len = 0;
+	    Strbuf_append1(tmp, (Char) c);
+	    Strbuf_terminate(tmp);
+	    histent->word = Strsave(tmp->s);
+	    return histent;
+	}
+    }
+    
+past:
+    switch (Stype) {
+
+    case TC_IF:
+	stderror(ERR_NAME | ERR_NOTFOUND, "then/endif");
+	break;
+
+    case TC_ELSE:
+	stderror(ERR_NAME | ERR_NOTFOUND, "endif");
+	break;
+
+    case TC_BRKSW:
+    case TC_SWITCH:
+	stderror(ERR_NAME | ERR_NOTFOUND, "endsw");
+	break;
+
+    case TC_BREAK:
+	stderror(ERR_NAME | ERR_NOTFOUND, "end");
+	break;
+
+    case TC_GOTO:
+	setname(short2str(Sgoal));
+	stderror(ERR_NAME | ERR_NOTFOUND, "label");
+	break;
+
+    default:
+	break;
+    }
+    /* NOTREACHED */
+    return NULL;
+}
+
 static int
 getword(struct Strbuf *wp)
 {
@@ -1723,8 +1856,17 @@ doumask(Char **v, struct command *c)
 #  endif
 # endif /* SYSVREL > 3 && BSDLIMIT */
 
-# if (defined(__linux__) || defined(__GNU__) || defined(__GLIBC__)) && defined(RLIMIT_AS) && !defined(RLIMIT_VMEM)
-#  define RLIMIT_VMEM	RLIMIT_AS
+# if (defined(__linux__) || defined(__GNU__) || defined(__GLIBC__))
+#  if defined(RLIMIT_AS) && !defined(RLIMIT_VMEM)
+#   define RLIMIT_VMEM	RLIMIT_AS
+#  endif
+/*
+ * Oh well, <asm-generic/resource.h> has it, but <bits/resource.h> does not
+ * Linux headers: When the left hand does not know what the right hand does.
+ */
+#  if defined(RLIMIT_RTPRIO) && !defined(RLIMIT_RTTIME)
+#   define RLIMIT_RTTIME (RLIMIT_RTPRIO + 1)
+#  endif
 # endif
 
 struct limits limits[] = 
@@ -1797,6 +1939,34 @@ struct limits limits[] =
     { RLIMIT_SBSIZE,	"sbsize",	1,	""		},
 # endif /* RLIMIT_SBSIZE */
 
+# ifdef RLIMIT_SWAP 
+    { RLIMIT_SWAP,	"swapsize",	1024,	"kbytes"	}, 
+# endif /* RLIMIT_SWAP */ 
+
+# ifdef RLIMIT_LOCKS 
+    { RLIMIT_LOCKS,	"maxlocks",	1,	""		}, 
+# endif /* RLIMIT_LOCKS */ 
+
+# ifdef RLIMIT_SIGPENDING 
+    { RLIMIT_SIGPENDING,"maxsignal",	1,	""		}, 
+# endif /* RLIMIT_SIGPENDING */ 
+
+# ifdef RLIMIT_MSGQUEUE 
+    { RLIMIT_MSGQUEUE,	"maxmessage",	1,	""		}, 
+# endif /* RLIMIT_MSGQUEUE */ 
+
+# ifdef RLIMIT_NICE 
+    { RLIMIT_NICE,	"maxnice",	1,	""		}, 
+# endif /* RLIMIT_NICE */ 
+
+# ifdef RLIMIT_RTPRIO 
+    { RLIMIT_RTPRIO,	"maxrtprio",	1,	""		}, 
+# endif /* RLIMIT_RTPRIO */ 
+
+# ifdef RLIMIT_RTTIME 
+    { RLIMIT_RTTIME,	"maxrttime",	1,	"usec"		}, 
+# endif /* RLIMIT_RTTIME */ 
+
     { -1, 		NULL, 		0, 	NULL		}
 };
 
@@ -2136,10 +2306,9 @@ void
 dosuspend(Char **v, struct command *c)
 {
 #ifdef BSDJOBS
-    int     ctpgrp;
     struct sigaction old;
 #endif /* BSDJOBS */
-    
+
     USE(c);
     USE(v);
 
@@ -2159,17 +2328,8 @@ dosuspend(Char **v, struct command *c)
 
 #ifdef BSDJOBS
     if (tpgrp != -1) {
-retry:
-	ctpgrp = tcgetpgrp(FSHTTY);
-	if (ctpgrp == -1)
+	if (grabpgrp(FSHTTY, opgrp) == -1)
 	    stderror(ERR_SYSTEM, "tcgetpgrp", strerror(errno));
-	if (ctpgrp != opgrp) {
-	    sigaction(SIGTTIN, NULL, &old);
-	    signal(SIGTTIN, SIG_DFL);
-	    (void) kill(0, SIGTTIN);
-	    sigaction(SIGTTIN, &old, NULL);
-	    goto retry;
-	}
 	(void) setpgid(0, shpgrp);
 	(void) tcsetpgrp(FSHTTY, shpgrp);
     }
