@@ -1,4 +1,4 @@
-/* $Header: /src/pub/tcsh/sh.proc.h,v 3.9 1997/10/27 22:44:32 christos Exp $ */
+/* $Header: /p/tcsh/cvsroot/tcsh/sh.proc.h,v 3.14 2010/05/07 18:16:07 christos Exp $ */
 /*
  * sh.proc.h: Process data structures and variables
  */
@@ -14,11 +14,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
+ * 3. Neither the name of the University nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -55,6 +51,7 @@ struct process {
     unsigned long p_flags;	/* various job status flags */
     unsigned char p_reason;	/* reason for entering this state */
     int     p_index;		/* shorthand job index */
+    pid_t   p_parentid;		/* parent pid */
     pid_t   p_procid;
     pid_t   p_jobid;		/* pid of job leader */
     /* if a job is stopped/background p_jobid gives its pgrp */
@@ -81,7 +78,7 @@ struct process {
 #  endif /* POSIX */
 # endif /* _SEQUENT_ */
 #endif /* BSDTIMES */
-    Char   *p_command;		/* first PMAXLEN chars of command */
+    Char   *p_command;		/* command */
 };
 
 /* flag values for p_flags */
@@ -107,8 +104,6 @@ struct process {
 #define PBACKQ		(1<<16)	/* Process is `` evaluation */
 #define PHUP		(1<<17)	/* Process is marked for SIGHUP on exit */
 
-#define	PMAXLEN		80
-
 /* defines for arguments to pprint */
 #define	NUMBER		01
 #define	NAME		02
@@ -120,7 +115,6 @@ struct process {
 #define	AREASON		0200
 
 EXTERN struct process proclist IZERO_STRUCT;/* list head of all processes */
-EXTERN bool    pnoprocesses IZERO;	/* pchild found nothing to wait for */
 
 EXTERN struct process *pholdjob IZERO;	/* one level stack of current jobs */
 
@@ -130,8 +124,4 @@ EXTERN struct process *pprevious IZERO;	/* previous job in table */
 
 EXTERN int   pmaxindex IZERO;		/* current maximum job index */
 
-#ifndef BSDTIMES
-EXTERN bool    timesdone;	/* shtimes buffer full ? */
-#endif /* BSDTIMES */
-
 #endif /* _h_sh_proc */
