@@ -1,4 +1,4 @@
-/* $Header: /p/tcsh/cvsroot/tcsh/tc.func.c,v 3.138 2008/01/24 20:17:22 christos Exp $ */
+/* $Header: /p/tcsh/cvsroot/tcsh/tc.func.c,v 3.143 2010/05/07 16:19:04 christos Exp $ */
 /*
  * tc.func.c: New tcsh builtins.
  */
@@ -32,7 +32,7 @@
  */
 #include "sh.h"
 
-RCSID("$tcsh: tc.func.c,v 3.138 2008/01/24 20:17:22 christos Exp $")
+RCSID("$tcsh: tc.func.c,v 3.143 2010/05/07 16:19:04 christos Exp $")
 
 #include "ed.h"
 #include "ed.defns.h"		/* for the function names */
@@ -118,7 +118,7 @@ expand_lex(const struct wordent *sp0, int from, int to)
 		 * elsewhere.
 		 */
 		if ((*s & QUOTE)
-		    && (((*s & TRIM) == HIST) ||
+		    && (((*s & TRIM) == HIST && HIST != '\0') ||
 			(((*s & TRIM) == '\'') && (prev_c != '\\')) ||
 			(((*s & TRIM) == '\"') && (prev_c != '\\')) ||
 			(((*s & TRIM) == '\\') && (prev_c != '\\')))) {
@@ -798,7 +798,7 @@ precmd(void)
     cleanup_push(&pintr_disabled, disabled_cleanup);
     if (precmd_active) {	/* an error must have been caught */
 	aliasrun(2, STRunalias, STRprecmd);
-	xprintf(CGETS(22, 3, "Faulty alias 'precmd' removed.\n"));
+	xprintf("%s", CGETS(22, 3, "Faulty alias 'precmd' removed.\n"));
 	goto leave;
     }
     precmd_active = 1;
@@ -816,7 +816,7 @@ postcmd(void)
     cleanup_push(&pintr_disabled, disabled_cleanup);
     if (postcmd_active) {	/* an error must have been caught */
 	aliasrun(2, STRunalias, STRpostcmd);
-	xprintf(CGETS(22, 3, "Faulty alias 'postcmd' removed.\n"));
+	xprintf("%s", CGETS(22, 3, "Faulty alias 'postcmd' removed.\n"));
 	goto leave;
     }
     postcmd_active = 1;
@@ -840,7 +840,7 @@ cwd_cmd(void)
     cleanup_push(&pintr_disabled, disabled_cleanup);
     if (cwdcmd_active) {	/* an error must have been caught */
 	aliasrun(2, STRunalias, STRcwdcmd);
-	xprintf(CGETS(22, 4, "Faulty alias 'cwdcmd' removed.\n"));
+	xprintf("%s", CGETS(22, 4, "Faulty alias 'cwdcmd' removed.\n"));
 	goto leave;
     }
     cwdcmd_active = 1;
@@ -862,7 +862,7 @@ beep_cmd(void)
     cleanup_push(&pintr_disabled, disabled_cleanup);
     if (beepcmd_active) {	/* an error must have been caught */
 	aliasrun(2, STRunalias, STRbeepcmd);
-	xprintf(CGETS(22, 5, "Faulty alias 'beepcmd' removed.\n"));
+	xprintf("%s", CGETS(22, 5, "Faulty alias 'beepcmd' removed.\n"));
     }
     else {
 	beepcmd_active = 1;
@@ -889,7 +889,7 @@ period_cmd(void)
     cleanup_push(&pintr_disabled, disabled_cleanup);
     if (periodic_active) {	/* an error must have been caught */
 	aliasrun(2, STRunalias, STRperiodic);
-	xprintf(CGETS(22, 6, "Faulty alias 'periodic' removed.\n"));
+	xprintf("%s", CGETS(22, 6, "Faulty alias 'periodic' removed.\n"));
 	goto leave;
     }
     periodic_active = 1;
@@ -927,7 +927,7 @@ job_cmd(Char *args)
     cleanup_push(&pintr_disabled, disabled_cleanup);
     if (jobcmd_active) {	/* an error must have been caught */
 	aliasrun(2, STRunalias, STRjobcmd);
-	xprintf(CGETS(22, 14, "Faulty alias 'jobcmd' removed.\n"));
+	xprintf("%s", CGETS(22, 14, "Faulty alias 'jobcmd' removed.\n"));
 	goto leave;
     }
     jobcmd_active = 1;
@@ -1035,7 +1035,7 @@ aliasrun(int cnt, Char *s1, Char *s2)
     cleanup_until(&w);
     pendjob();
     /* Restore status */
-    setv(STRstatus, putn(status), VAR_READWRITE);
+    setv(STRstatus, putn((tcsh_number_t)status), VAR_READWRITE);
 }
 
 void
@@ -1134,7 +1134,7 @@ rmstar(struct wordent *cp)
 		    if (!Strcmp(args->word, STRstar))
 			star = 1;
 		if (ask && star) {
-		    xprintf(CGETS(22, 8,
+		    xprintf("%s", CGETS(22, 8,
 			    "Do you really want to delete all files? [n/y] "));
 		    flush();
 		    (void) force_read(SHIN, &c, 1);
@@ -1928,7 +1928,7 @@ getremotehost(int dest_fd)
 				(ptr = strstr(domain, s)) != NULL) {
 			        char *cbuf;
 
-				cbuf = strspl(name, ptr);
+				cbuf = strspl(name, ptr + strlen(s));
 				if (getaddrinfo(cbuf, NULL, &hints, &res) != 0)
 				    res = NULL;
 				xfree(cbuf);
